<?php
/**
 * @author Takács Ákos (Rimelek), programmer [at] rimelek [dot] hu
 * @copyright Copyright (C) 2009, Takács Ákos
 * @package REPHPGraph
 */

/**
 * Elipszis iterátor
 * 
 * Ez az osztály segítséget nyújt az elipszis formában történő iterációhoz.<br />
 * Csupán a kezdő és befejező szögelfordulás fokát kell megadni,
 * valamint hol legyen az elipszis centruma, mi legyen a szélessége és magassága, 
 * végül hogy hány fokonként iteráljon. Ezek után az iterátor kiszámolja az X és Y 
 * koordinátákat minden ciklusban, és visszaadja azt. 
 * 
 * {@example ../examples/ellipseIterator.php}
 * 
 * @author Takács Ákos (Rimelek), programmer [at] rimelek [dot] hu
 * @copyright Copyright (C) 2009, Takács Ákos
 * @version 1.0
 * @package REPHPGraph
 */
class EllipseIterator implements Iterator, ArrayAccess 
{
	/**
	 * Iterátor centrumának X koordinátája
	 * 
	 * @var int
	 */
	protected $cx = 0;
	/**
	 * Iterátor centrumának Y koordinátája
	 * 
	 * @var int
	 */
	protected $cy = 0;
	/**
	 * Az iteráció aktuális szöge
	 * 
	 * CSak olvasható (readonly)
	 * 
	 * @var int
	 */
	protected $_i = 1;
	/**
	 * Elipszis magassága
	 * @var int
	 */
	protected $height = 100;
	/**
	 * Elipszis szélessége
	 * @var int
	 */
	protected $width = 100;
	/**
	 * Kezdő szög
	 * @var int
	 */
	protected $start = 0;
	/**
	 * Befejező szög
	 * @var int
	 */
	protected $end = 360;
	/**
	 * Iteráció lépésköze fok-ban számítva
	 * @var int
	 */
	protected $add = 1;
			
	protected $key = 0;
	protected $valid=true;
	/**
	* Tulajdonságok értékeinek lekérdezésekor lefutó metódus
	* 
	* @param string $var
	* @return mixed
	*/
	function __get($var) 
	{	
		return $this->$var;
	}
	
	/**
	* Tulajdonságok értékeinek beállításakor lefutó metódus
	* 
	* @param string $var
	* @param mixed $value
	*/
	function __set($var,$value)
	{
		$value = (int)$value;
		switch ($var)
		{
			case 'height': case 'width': case 'end': case 'key':
				$this->$var = $value; 
				break;
			case 'start':
				$this->_i = $value;
				$this->start = $value;
				break;
			case 'add':
				$this->add = ($value <= 0) ? 1 : $value;
				break;                          
		}
		$this->$var = $value;
	}
	/**
	* EllipseIterator konstruktor
	* 
	* Csak értékeket állít be, amit majd az iterációkor hazsnál  
	* 
	* @param int $cx Elipszis centrumának X koordinátája
	* @param int $cy Elipszis centrumának Y koordinátája
	* @param int $width Elipszis szélessége
	* @param int $height Elipszis magassága
	*/
	function __construct($cx,$cy,$width,$height)
	{       
		$this->_i = 0;
		$this->width = $width;
		$this->height = $height;
		$this->cx = $cx;
		$this->cy = $cy;

	}
	
	/**
	 * Iteráció alaphelyzetbe állítása
	 *
	 */
	public function rewind() 
	{ 
		$this->_i = $this->start;
		$this->key=0;
		$this->valid=true;
	}
	
	/**
	 * Aktuális elemek visszaadása
	 *
	 * @return array
	 */
	public function current() 
	{ 
		$x = $this->cx + (cos(deg2rad($this->_i)) * ceil($this->width / 2));
		$y = $this->cy + (sin(deg2rad($this->_i)) * ceil($this->height / 2));  
		return array('x'=>$x,'y'=>$y);
	}
	
	/**
	 * Következő elem előállítása
	 *
	 */
	public function next() 
	{ 
		$this->key++;
		if ($this->start <= $this->end) {
			$this->_i += $this->add;
			if ($this->_i <= $this->end) {
				$this->valid = true;
				return;
			} 
			$this->_i = 1;
			$this->valid = false;
		} else {
			$this->_i -= $this->add;
			if ($this->_i >= $this->end) {
				$this->valid = true;
				return;
			} 
			$this->_i = 1;
			$this->valid = false;      
		} 
	}
	
	/**
	 * Aktuális index visszaadása
	 *
	 * @return int
	 */
	public function key() 
	{
		return $this->key;		
	}
	
	/**
	 * Van-e még több elem
	 *
	 * @return boolean
	 */
	public function valid() 
	{   
		return $this->valid;
	}
	
	/**
	 * Létezik-e egy $offset indexű elem
	 *
	 * @param int $offset
	 * @return boolean
	 */
	public function offsetExists($offset) 
	{
		return (abs(abs($this->start) - abs($this->end)) / $this->add) >= $offset;
	}
	
	/**
	 * Konkrét elem elérése index alapján
	 *
	 * @param int $offset
	 * @return array
	 */
	public function offsetGet($offset) 
	{
		$x='';
		$y='';
		if ($this->offsetExists($offset)) {
			$i = $this->start+$offset*$this->add;
			$x = $this->cx + (cos(deg2rad($i)) * ceil($this->width / 2));
			$y = $this->cy + (sin(deg2rad($i)) * ceil($this->height / 2)); 
		}
		return array('x'=>$x,'y'=>$y);
	}
	
	/**
	 * $offset indexű elem beállítása
	 *
	 * A beállítás nem lehetséges!
	 * Csak az adott indexű elem x és y koordinátájának megváltoztatása
	 * 
	 * @param int $offset
	 * @param mixed $value
	 */
	public function offsetSet($offset,$value) 
	{
		throw new Exception("Az értékek nem állíthatók be közvetlenül az iterátorokban!");	
	}
	
	/**
	 * Elem megszüntetése
	 *
	 * Nem lehetséges!
	 * 
	 * @param mixed $offset
	 */
	public function offsetUnset($offset)
	{
		throw new Exception("Az iterátor elemei nem szüntethetők meg!");	
	}
}
?>