<?php 
/**
 * @author Takács Ákos (Rimelek), programmer [at] rimelek [dot] hu
 * @copyright Copyright (C) 2009, Takács Ákos
 * @package REPHPGraph
 */

/**
 * Aobject importálása
 */
require_once dirname(__FILE__)."/../AObject.class.php";

/**
 * IObject importálása
 */
require_once dirname(__FILE__)."/../interfaces/IObject.class.php";

/**
 * Ellipszis iterátor importálása
 */
require_once dirname(__FILE__)."/../iterators/MultiEllipseIterator.class.php";

/**
 * Analóg óra
 *
 * Egy analóg órát lehet létrehozni vele. A kinézetét a többi objektumhoz hasonlóan módosítani lehet. 
 *
 * {@example ../examples/analogclock.php}
 *
 * @property $hourHandColor
 * @property $minuteHandColor
 * @property $secondHandColor
 * @property $segmentationColor 
 *
 * @version 1.0
 * @package REPHPGraph
 */
class AnalogClock extends AObject implements IObject
{
	/**
	 * A megjelenített időpont elemei
	 * 
	 * <b>indexek:<b><br />
	 * <ul>
	 * 	<li>hour12: Az óra 12 órás formátumban</li>
	 * 	<li>hour24: Az óra 24 órás formátumban</li>
	 *	<li>minute: perc</li>
	 * 	<li>second: másodperc</li>
	 * 	<li>am_pm: Délelőtt, vagy délután</li>
	 * </ul>
	 *
	 * @var array
	 */
	protected $time = array();
	
	/**
	 * A használt színek
	 *
	 * @var array
	 */
	protected $colors = array();
	
	/**
	 * Az óra sugara
	 *
	 * @var int
	 */
	protected $radius=0;

	/**
	 * Óramutató aránya a sugárhoz képest százalékban
	 *
	 * @var int
	 */
	protected $hourHandRatio=50;
	
	/**
	 * Percmutató aránya a sugárhoz képest százalékban
	 *
	 * @var int
	 */
	protected $minuteHandRatio=70;
	
	/**
	 * Másodpercmutató aránya a sugárhoz képest százalékban
	 *
	 * @var int
	 */
	protected $secondHandRatio=80;
	
	/**
	 * Óra létrehozása
	 *
	 * @param int $radius Az óra sugara
	 * @param int $time A megjelenítendő időpont (timestamp). Elhagyása esetén az aktuális idő 
	 */
	function __construct($radius,$time=null)
	{
		$this->width = $this->height = ($this->radius = $radius)*2;
		$time = ($time !== null) ? $time : time(); 
		
		//színek beállítása
		$this->time['hour12'] = date("h",$time);
		$this->time['hour24'] = date("H",$time);
		$this->time['minute']  = date("i",$time);
		$this->time['second'] = date("s",$time);
		$this->time['am_pm'] = date("A",$time);

		$this->colors['borderColor']		= array(0,0,1);			//óralap szegélye
		$this->colors['color']			= array(255,255,254);		//óralap színe
		$this->colors['backGround']		= array(255,255,255);		//háttér
		$this->colors['hourHandColor']		= array(200,10,60);		//óra mutató színe
		$this->colors['minuteHandColor']	= array(100,200,250);		//perc mutató színe
		$this->colors['secondHandColor']	= array(20,200,30);		//másodperc mutató színe
		$this->colors['segmentationColor']	= array(0,0,0);			//Az órán a beosztások, és a középont színe
		//kép létrehozása     
		$this->source = imagecreatetruecolor($this->width,$this->height);
		$this->setColors();
	}
    
	/**
	 * Tulajdonságok lekérdezésének szabályozása
	 *
	 * @param string $var
	 * @return mixed
	 */
	public function __get($var) 
	{
		if (isset($this->colors[$var])) return $this->colors[$var];
		return parent::__get($var);
	}
	
	/**
	 * Tulajdonságok beállításának szabályozása
	 * 
	 * @param string $var
	 * @param mixed $value
	 */
	public function __set($var,$value) 
	{
		if (isset($this->colors[$var])) $this->colors[$var] = $value;
		else parent::__set($var,$value);
	}
	
	/**
	 * Színek beállítása
	 *
	 */
	protected function setColors()
	{
		foreach($this->colors as $key => $value)
		{ 
			$this->$key = $this->createColor($value[0],$value[1],$value[2]);
		}	
	}

	/**
	 * Az óralap nem mozgó elemeinek kirajzolása
	 *
	 */
	protected function dial()
	{
		//Az óralapot határoló kör létrehozása
		ImageArc($this->source,
			round($this->width/2),round($this->height/2),
			$this->width-1,$this->height-1,0,360,$this->borderColor);
		//Az óra, és perc beosztások kirajzolása
		$this->segmentation();
		//A 'délután' (PM), vagy 'délelőtt' (AM) jelzése
		ImageString($this->source,2,round($this->width/2)-3,round($this->height/4),$this->time['am_pm'],$this->segmentationColor);
	}

	/**
	 * Óra-, és percosztások kirajzolása
	 *
	 */
	protected function segmentation()
	{
		//4 elipszis ( tömött karikák,vonalak belső vége, vonalak külső vége, számok )
		// középpontjának x, és y koordinátája 
		$cx = array($this->width/2 ,$this->width/2,$this->width/2 ,$this->width/2);
		$cy = array($this->height/2,$this->height/2,$this->height/2,$this->height/2);
		//a fenti 4 elipszis szélessége , és magassága
		$width = array($this->width-10,$this->width-4,$this->width-20,$this->width-34);
		$height = array($this->height-10,$this->height-4,$this->height-20,$this->height-34);
		
		//Iterátor létrehozása több elipszis koordinátáival
		$iterator = new MultiEllipseIterator($cx,$cy,$width,$height);
		$iterator->start = -60; //iteráció kezdete fokban
		$iterator->end = 299; //iteráció vége
		$iterator->add = 6; //Lépésköz

		//Az óralap
		$hour = 1;
		foreach($iterator as $k=>$position) { 
			//A perc osztások 
			if(($iterator->_i % 30)){
				ImageFilledArc($this->source, round($position['x'][0]), round($position['y'][0]), 3,3, 0, 360, $this->segmentationColor, IMG_ARC_PIE);
			//Az óra osztások
			}else{ 
				imageLine($this->source, round($position['x'][1]), round($position['y'][1]), round($position['x'][2]), round($position['y'][2]), $this->segmentationColor);
				//Az óra osztások mellett az óra száma
				ImageString($this->source,2,round($position['x'][3])-3,round($position['y'][3])-5,$hour,$this->segmentationColor);
				$hour++; 
			}
		}
	}

	/**
	 * Mutatók kirajzolása
	 */
	protected function hands()
	{
		$this->hourHand();	//óra
		$this->minuteHand();	//perc
		$this->secondHand();	//másodperc
	}

	/**
	 * Óramutató létrehozása
	 *
	 * @param double $arany Óramutató aránya a sugárhoz képest
	 */
	protected function hourHand()
	{
		$ratio = $this->hourHandRatio/100;
		//óra mutató poziciója
		$hourDeg = deg2rad( 30 * $this->time['hour12'] ) - deg2rad(90) + deg2rad($this->time['minute']/2);
		$hourX = (($this->radius*$ratio) * cos($hourDeg)) + $this->radius;
		$hourY = (($this->radius*$ratio) * sin($hourDeg)) + $this->radius;

		//óra mutató
		imageLine($this->source, round($this->width/2), round($this->height/2), round($hourX), round($hourY), $this->hourHandColor);
	}

	/**
	 * Percmutató létrehozása
	 *
	 * @param double $arany Percmutató aránya a sugárhoz képest
	 */	
	protected function minuteHand()
	{
		$ratio = $this->minuteHandRatio/100;
		//a perc mutató poziciója
		$minuteDeg = deg2rad( 6 * $this->time['minute'] ) - deg2rad(90) + deg2rad($this->time['second']/10);
		$minuteX = (($this->radius*$ratio) * cos($minuteDeg)) + $this->radius;
		$minuteY = (($this->radius*$ratio) * sin($minuteDeg)) + $this->radius;
        
		//perc mutató
		imageLine($this->source, round($this->width/2), round($this->height/2), round($minuteX), round($minuteY), $this->minuteHandColor);
	}
    
	/**
	 * Másodpercmutató létrehozása
	 *
	 * @param double $arany Másodpercmutató aránya a sugárhoz képest
	 */	
	protected function secondHand()
	{
		$ratio = $this->secondHandRatio/100;
		//a másodperc poziciójának számítása
		$secondDeg = deg2rad( 6 * $this->time['second'] ) - deg2rad(90);
		$secondX = (($this->radius*$ratio) * cos($secondDeg)) + $this->radius;
		$secondY = (($this->radius*$ratio) * sin($secondDeg)) + $this->radius;

		//Másodperc mutató
		imageLine($this->source, round($this->width/2), round($this->height/2), round($secondX), round($secondY), $this->secondHandColor);	
	}
    
	/**
	 * A teljes óra kép létrehozáa
	 *
	 */
	public function create()
	{
		//egyező háttér és szegélyszín esetén új keretszín választása
		$color = $this->createColor(1,1,1);
		if ($this->borderColor == $this->backGround) {
			if ($this->borderColor == $color) {
				$this->borderColor = $this->createColor(0,0,0);
			} else {
				$this->borderColor = $color;
			}
		}

		//óralap színének beállítása
		imagefill($this->source,0,0,$this->color);
		//Óralap kirajzolása
		$this->dial();
		//Mutatók kirajzolása
		$this->hands();
		//háttér beállítás
		imagefill($this->source,0,0,$this->backGround);
		imagefill($this->source,0,$this->height-1,$this->backGround);
		imagefill($this->source,$this->width-1,$this->height-1,$this->backGround);
		imagefill($this->source,$this->width-1,0,$this->backGround);
		//a középpontban egy tömött karika
		ImageFilledArc($this->source, round($this->width/2), round($this->height/2), 5,5, 0, 360, $this->segmentationColor, IMG_ARC_PIE);
	}
    
}

?>
