<?php
/**
 * @author Takács Ákos (Rimelek), programmer [at] rimelek [dot] hu
 * @copyright Copyright (C) 2008, Takács Ákos
 * @package REPHPGraph
 */

/**
 * Aobject importálása
 */
require_once dirname(__FILE__)."/../AObject.class.php";

/**
 * IObject importálása
 */
require_once dirname(__FILE__)."/../interfaces/IObject.class.php";

/**
 * FileExistsException ipmortálása
 */
require_once dirname(__FILE__)."/../exceptions/FileExistsException.class.php";

/**
 * Szövegeket kezelő osztály
 * 
 * Ez az osztály végzi a szövegek létrehozását. A szöveg betűmérete és típusa is megadható.
 * Alaméprtelmezett típus az <b>Arial</b><br /><br />
 * 
 * Tipp: A {@link $backGround} tulajdonságot a jó minőség érdekében állítsuk 
 * azonosra a kép hátterének színével Akkor is, ha a szöveget transparent-re állítjuk!<br /><br /> 
 * 
 * Egy egyszerű szöveget a következő módon hozhatunk létre:<br />
 * {@example ../examples/text.php}
 * 
 * @version 1.0
 * @package REPHPGraph
 */
class Text extends AObject implements IObject
{
	/**
	 * Szövegegdoboz koordinátái
	 * 
	 * {@link http://php.net/manual/hu/function.imagettfbbox.php}<br />
	 * Csak olvasható (readonly)
	 * 
	 * @var array
	 */
	protected $ttfbox = array();
        
	/**
	 * Betűtípus
	 * 
	 * Alapértelmezett az Arial, de beállítható saját betűtípus is az url megadásával<br />
	 * Csak olvasható (readonly)
	 * 
	 * @var string
	 */
	protected $fonttype = './fonts/arial.ttf';
	/**
	 * Betűméret
	 * 
	 * Csak olvasható (readonly)
	 * 
	 * @var int
	 */
	protected $fontsize = 10;
	/**
	 * Szöveg tartalma
	 * 
	 * Csak olvasható (readonly)
	 * 
	 * @var string
	 */
	protected $value = '';
	
	/**
	 *  A képméret kiszámításához szükséges tulajdonságokat csak konstruktorban lehet
	 * beállítani. Később módosítani nem lehet.
	 * 
	 * @param string $text
	 * @param int $fontsize
	 * @param int $angle
	 * @param string $fonttype
	 */        
	function Text($text,$fontsize=20,$angle=0,$fonttype = null)
	{
		//Elforgatás átváltása -180 -s +180 fok közé
		if (($angle = $angle % 360) > 180) $angle = $angle-360;
		//Alapértelmezett fonttípus beállítása
		if (!$fonttype) { $fonttype = dirname(dirname(__FILE__)) . ltrim($this->fonttype,'.'); }
		//szöveget körülvevő szövegdoboz koordinátái
		$this->ttfbox = $ttfbox = imageTTFBBox($fontsize,-$angle,$fonttype,$text);
		//szélesség és magasság kiszámítása
		$width = abs($ttfbox[0]) + abs($ttfbox[4]);
		$height = abs($ttfbox[1]) + abs($ttfbox[5]);
		$height = ($height < ($maxY = (abs($ttfbox[3]) + abs($ttfbox[7]))))
			? $maxY : $height;
		$width = ($width < ($maxX = (abs($ttfbox[2]) + abs($ttfbox[6]))))
			? $maxX : $width;
				
		$this->width = $width;
		$this->height = $height;
                
		$this->angle = $angle;
		$this->fonttype = $fonttype;
		$this->fontsize = $fontsize;
                
		//Hibás betűtípus esetén kivétel kiváltása 
		if (!file_exists($this->fonttype)) {
			throw new FileExistsException("A betűtípus fájl nem található!");
		}
                
		$this->value = $text;
		//kép létrehozása
		$this->source = imagecreatetruecolor($width,$height);
		//színek beállítása
		$this->backGround = imageColorAllocate($this->source,1,1,1);
		$this->color = imageColorAllocate($this->source,1,1,1);
		$this->borderColor = null;
	}
	/**
	 * X koordináta beállítása
	 * 
	 * $Jx paraméter beállításával megszabható, hogy az X koordinátát a szöveg mely pontjától számolja<br />
	 * <b>$Jx értékei</b><br/>
	 * <ul>
	 * <li>0: Szöveg bal oldalától</li>
	 * <li>1: szöveg közepétől</li>
	 * <li>2: szöveg jobb oldalától</li>
	 * </ul>
	 * @param int $x
	 * @param int $Jx 0 | 1 | 2
	 */
	function setX($x,$Jx = 0)
	{
		if($Jx == 0) {
			$eltol = ceil($this->width / 2);
		} else if ($Jx == 1) {
			$eltol = 0;
		} else {
			$eltol = 0-ceil($this->width / 2);
		}
		$this->x = $eltol + $x;
	}
	/**
 	 * Y koordináta beállítása
	 * 
	 * $Jy paraméter beállításával megszabható, hogy az Y koordinátát a szöveg mely pontjától számolja<br />
	 * <b>$Jy értékei</b><br/>
	 * <ul>
	 * <li>0: Szöveg tetejétől</li>
	 * <li>1: szöveg közepétől</li>
	 * <li>2: szöveg aljától</li>
	 * </ul> 
	 * 
	 * @param int $y
	 * @param int $Jy 0 | 1 | 2
	 */
	function setY($y,$Jy = 0)
	{
		if($Jy == 0) {
			$eltol = ceil($this->height / 2);
		} else if ($Jy == 1) {
			$eltol = 0;
		} else {
			$eltol = 0-ceil($this->height / 2);
		}
		$this->y = $eltol + $y;
	}

 	/**
 	 * Szöveg létrehozása a képen
 	 * 
 	 */
	function create()
	{
		if ($this->angle < 0) {
			$y = $this->height-$this->ttfbox[1];
			if (abs($this->angle) <= 90)  {
				$x = abs($this->ttfbox[6]);
			} else {
				$x = abs($this->ttfbox[4]);
				$y = $this->height - $this->ttfbox[7];
			}
		} else {
			$y = $this->height - $this->ttfbox[3];
			if ($this->angle <= 90) {
				$x = $this->width - $this->ttfbox[4];
			} else {
				$x = $this->width - $this->ttfbox[6];
				$y=abs($this->ttfbox[1]);
			}
		}
		imageTTFText($this->source,$this->fontsize,-$this->angle,$x,$y,$this->color,$this->fonttype,$this->value);	
		if ($this->borderColor != $this->backGround and $this->borderColor) {
			$this->createBorder($x,$y);
		}
	}
 	/**
 	 * Szöveg körüli keret 
	  * 
 	 * ($x,$y) koordináták a keret bal alsó koordinátái. 
 	 * A szélesség a szöveg szélessége lesz, magassága pedig a szöveg magassága. 
 	 * 
 	 * @param int $x
 	 * @param int $y
 	 */
	protected function createBorder($x,$y)
	{
		$leftTopX = $x + $this->ttfbox[6];
		$leftTopY = $y + $this->ttfbox[7];
		$leftBottomX = $x + $this->ttfbox[0];
		$leftBottomY = $y + $this->ttfbox[1];
           
		$rightTopX = $x + $this->ttfbox[4];
		$rightTopY = $y + $this->ttfbox[5];
		$rightBottomX = $x + $this->ttfbox[2];
		$rightBottomY = $y + $this->ttfbox[3];
   
		//keret széleségének és magasságának 1 px-el csökkentése
		$leftBottomY += ($this->angle > -135 && $this->angle <= 45) ? -1 : 1;
		$leftBottomX += (($this->angle >= 135 && $this->angle <= 180) 
				or ($this->angle < -45 && $this->angle >= -180)) ? -1 : 1;
		$leftTopX += (($this->angle > 0 && $this->angle < 45)
				or ( $this->angle > 45 && $this->angle <= 180 )
				or $this->angle < -135) ? -1 : 1;
		$leftTopY += (($this->angle > 135 && $this->angle <= 180)
				or ( $this->angle <= -45 && $this->angle >= -180 )) ? -1 : 1;
                        
		$rightBottomX += ($this->angle > -135 && $this->angle <= 45) ? -1 : 1;
		$rightBottomY += ($this->angle > -45 && $this->angle <= 135) ? -1 : 1;
		$rightTopX += ($this->angle >= -45 && $this->angle <= 135) ? -1 : 1;
		$rightTopY += (($this->angle > 45 && $this->angle <= 180)
				or $this->angle < -135) ? -1 : 1;               
                
		//bal oldali keret
		imageline($this->source,
			$leftBottomX,$leftBottomY,$leftTopX,$leftTopY,
			$this->borderColor);
		//jobb oldali keret
		imageline($this->source,
			$rightBottomX,$rightBottomY,$rightTopX,$rightTopY,
			$this->borderColor);
		//felső keret
		imageline($this->source,
			$leftTopX,$leftTopY,$rightTopX,$rightTopY,
			$this->borderColor);
		//alsó keret
		imageline($this->source,
			$leftBottomX,$leftBottomY,$rightBottomX,$rightBottomY,
			$this->borderColor);     
	}	
}
?>